@@ -20,6 +20,9 @@
 #' @return A 1d atomic vector or a data frame.
 #' @keywords internal
 #'
+#' @section Dependencies:
+#' - [vec_proxy()] called by default
+#'
 #' @export
 vec_proxy_equal <- function(x, ...) {
   if (!missing(...)) {
@@ -48,6 +51,12 @@ vec_proxy_equal.default <- function(x, ...) {
 #'
 #' @inheritParams vec_compare
 #' @return A logical vector the same size as. Will only contain `NA`s if `na_equal` is `FALSE`.
+#'
+#' @section Dependencies:
+#' - [vec_cast_common()] with fallback
+#' - [vec_recycle_common()]
+#' - [vec_proxy_equal()]
+#'
 #' @export
 #' @examples
 #' vec_equal(c(TRUE, FALSE, NA), FALSE)
@@ -66,7 +75,7 @@ vec_equal <- function(x, y, na_equal = FALSE, .ptype = NULL) {
   args <- vec_cast_common_params(
     !!!args,
     .to = .ptype,
-    .df_fallback = DF_FALLBACK_QUIET
+    .df_fallback = DF_FALLBACK_quiet
   )
   .Call(vctrs_equal, args[[1]], args[[2]], na_equal)
 }
