@@ -4,6 +4,7 @@
 #include "ptype-common.h"
 #include "slice-assign.h"
 #include "type-data-frame.h"
+#include "owned.h"
 #include "utils.h"
 
 
@@ -180,7 +181,7 @@ static SEXP vec_rbind(SEXP xs,
     init_compact_seq(idx_ptr, counter, size, true);
 
     // Total ownership of `out` because it was freshly created with `vec_init()`
-    out = df_assign(out, idx, x, vctrs_ownership_total, &bind_assign_opts);
+    out = df_assign(out, idx, x, VCTRS_OWNED_true, &bind_assign_opts);
     REPROTECT(out, out_pi);
 
     if (has_rownames) {
@@ -198,7 +199,7 @@ static SEXP vec_rbind(SEXP xs,
       PROTECT(rn);
 
       if (rownames_type(rn) == ROWNAMES_IDENTIFIERS) {
-        rownames = chr_assign(rownames, idx, rn, vctrs_ownership_total);
+        rownames = chr_assign(rownames, idx, rn, VCTRS_OWNED_true);
         REPROTECT(rownames, rownames_pi);
       }
 
@@ -247,7 +248,7 @@ static SEXP vec_rbind(SEXP xs,
     }
   }
 
-  out = vec_restore(out, ptype, PROTECT(r_int(n_rows)));
+  out = vec_restore(out, ptype, PROTECT(r_int(n_rows)), VCTRS_OWNED_true);
 
   UNPROTECT(n_prot);
   UNPROTECT(4);
@@ -460,12 +461,12 @@ static SEXP vec_cbind(SEXP xs, SEXP ptype, SEXP size, struct name_repair_opts* n
     init_compact_seq(idx_ptr, counter, xn, true);
 
     // Total ownership of `out` because it was freshly created with `Rf_allocVector()`
-    out = list_assign(out, idx, x, vctrs_ownership_total);
+    out = list_assign(out, idx, x, VCTRS_OWNED_true);
     REPROTECT(out, out_pi);
 
     SEXP xnms = PROTECT(r_names(x));
     if (xnms != R_NilValue) {
-      names = chr_assign(names, idx, xnms, vctrs_ownership_total);
+      names = chr_assign(names, idx, xnms, VCTRS_OWNED_true);
       REPROTECT(names, names_pi);
     }
     UNPROTECT(1);
@@ -480,7 +481,7 @@ static SEXP vec_cbind(SEXP xs, SEXP ptype, SEXP size, struct name_repair_opts* n
     Rf_setAttrib(out, R_RowNamesSymbol, rownames);
   }
 
-  out = vec_restore(out, type, R_NilValue);
+  out = vec_restore(out, type, R_NilValue, VCTRS_OWNED_true);
 
   UNPROTECT(9);
   return out;
