@@ -379,7 +379,7 @@ static ptrdiff_t suffix_pos(const char* name) {
       goto done;
 
     default:
-      Rf_error("Internal error: Unexpected state in `suffix_pos()`");
+      stop_internal("suffix_pos", "Unexpected state.");
     }}
 
  done:
@@ -469,10 +469,10 @@ static void describe_repair(SEXP old_names, SEXP new_names) {
 // [[ register() ]]
 SEXP vctrs_outer_names(SEXP names, SEXP outer, SEXP n) {
   if (names != R_NilValue && TYPEOF(names) != STRSXP) {
-    Rf_error("Internal error: `names` must be `NULL` or a string");
+    stop_internal("vctrs_outer_names", "`names` must be `NULL` or a string.");
   }
   if (!r_is_number(n)) {
-    Rf_error("Internal error: `n` must be a single integer");
+    stop_internal("vctrs_outer_names", "`n` must be a single integer.");
   }
 
   if (outer != R_NilValue) {
@@ -488,7 +488,7 @@ SEXP outer_names(SEXP names, SEXP outer, R_len_t n) {
     return names;
   }
   if (TYPEOF(outer) != CHARSXP) {
-    Rf_error("Internal error: `outer` must be a scalar string.");
+    stop_internal("outer_names", "`outer` must be a scalar string.");
   }
 
   if (outer == strings_empty || outer == NA_STRING) {
@@ -523,7 +523,7 @@ SEXP apply_name_spec(SEXP name_spec, SEXP outer, SEXP inner, R_len_t n) {
     return inner;
   }
   if (TYPEOF(outer) != CHARSXP) {
-    Rf_error("Internal error: `outer` must be a scalar string.");
+    stop_internal("apply_name_spec", "`outer` must be a scalar string.");
   }
 
   if (outer == strings_empty || outer == NA_STRING) {
@@ -870,7 +870,7 @@ SEXP vctrs_validate_minimal_names(SEXP names, SEXP n_) {
 
   if (TYPEOF(n_) == INTSXP) {
     if (Rf_length(n_) != 1) {
-      Rf_error("Internal error (minimal names validation): `n` must be a single number.");
+      stop_internal("vctrs_validate_minimal_names", "`n` must be a single number.");
     }
     n = INTEGER(n_)[0];
   }
