@@ -240,11 +240,11 @@ test_that("vec_compare() silently falls back to base data frame", {
 # order/sort --------------------------------------------------------------
 
 test_that("can request NAs sorted first", {
-  expect_equal(vec_order(c(1, NA), "asc", "large"), 1:2)
-  expect_equal(vec_order(c(1, NA), "desc", "large"), 2:1)
+  expect_equal(vec_order(c(1, NA), "asc", "largest"), 1:2)
+  expect_equal(vec_order(c(1, NA), "desc", "largest"), 2:1)
 
-  expect_equal(vec_order(c(1, NA), "asc", "small"), 2:1)
-  expect_equal(vec_order(c(1, NA), "desc", "small"), 1:2)
+  expect_equal(vec_order(c(1, NA), "asc", "smallest"), 2:1)
+  expect_equal(vec_order(c(1, NA), "desc", "smallest"), 1:2)
 })
 
 test_that("can sort data frames", {
