@@ -212,6 +212,17 @@ test_that("vec_match() matches match()", {
   expect_equal(vec_match("x", "x"), match("x", "x"))
 })
 
+test_that("vec_match() and vec_in() check types", {
+  verify_errors({
+    df1 <- data_frame(x = data_frame(foo = 1))
+    df2 <- data_frame(x = data_frame(foo = ""))
+    expect_error(vec_match(df1, df2), class = "vctrs_error_incompatible_type")
+    expect_error(vec_match(df1, df2, needles_arg = "n", haystack_arg = "h"), class = "vctrs_error_incompatible_type")
+    expect_error(vec_in(df1, df2), class = "vctrs_error_incompatible_type")
+    expect_error(vec_in(df1, df2, needles_arg = "n", haystack_arg = "h"), class = "vctrs_error_incompatible_type")
+  })
+})
+
 test_that("vec_in() matches %in%", {
   n <- c(1:3, NA)
   h <- c(4, 2, 1, NA)
@@ -302,3 +313,26 @@ test_that("missing values are propagated across columns", {
 test_that("can't supply NA as `na_equal`", {
   expect_error(vec_match(NA, NA, na_equal = NA), "single `TRUE` or `FALSE`")
 })
+
+test_that("dictionary tools have informative errors", {
+  verify_output(test_path("error", "test-dictionary.txt"), {
+    "# vec_match() and vec_in() check types"
+    df1 <- data_frame(x = data_frame(foo = 1))
+    df2 <- data_frame(x = data_frame(foo = ""))
+    vec_match(df1, df2)
+    vec_match(df1, df2, needles_arg = "n", haystack_arg = "h")
+    vec_in(df1, df2)
+    vec_in(df1, df2, needles_arg = "n", haystack_arg = "h")
+  })
+})
+
+test_that("vec_match() and vec_in() silently fall back to base data frame", {
+  expect_silent(expect_identical(
+    vec_match(foobar(mtcars), foobar(tibble::as_tibble(mtcars))),
+    1:32
+  ))
+  expect_silent(expect_identical(
+    vec_in(foobar(mtcars), foobar(tibble::as_tibble(mtcars))),
+    rep(TRUE, 32)
+  ))
+})
