@@ -2,7 +2,7 @@ context("test-type-bare")
 
 
 test_that("ptype2 base methods are not inherited", {
-  ptypes <- vec_remove(base_empty_types, "null")
+  ptypes <- vec_remove(base_empty_types, c("null", "dataframe"))
   for (ptype in ptypes) {
     x <- new_vctr(ptype, class = "foobar", inherit_base_type = TRUE)
     expect_is(vec_ptype2(x, x), "foobar")
@@ -12,17 +12,20 @@ test_that("ptype2 base methods are not inherited", {
 })
 
 test_that("cast base methods are not inherited", {
-  # FIXME: Should also disallow data frame and list methods
-  ptypes <- vec_remove(base_empty_types, c("null", "dataframe", "list"))
+  ptypes <- vec_remove(base_empty_types, c("null", "dataframe"))
   for (ptype in ptypes) {
     x <- new_vctr(ptype, class = "foobar", inherit_base_type = TRUE)
     expect_is(vec_cast(ptype, x), "foobar")
-    expect_error(vec_cast(x, !!ptype), class = "vctrs_error_incompatible_cast")
+    expect_error(vec_cast(x, ptype), class = "vctrs_error_incompatible_type")
   }
 })
 
+test_that("default cast allows objects with the same type", {
+  x <- structure(1, class = c("foo", "double"))
+  expect_equal(vec_cast(x, x), x)
+})
 
-# shape_match -------------------------------------------------------------
+# vec_shaped_ptype -------------------------------------------------------
 
 test_that("array dimensions are preserved", {
   mat1 <- matrix(lgl(), nrow = 1, ncol = 1)
@@ -31,15 +34,15 @@ test_that("array dimensions are preserved", {
 
   expect_equal(vec_ptype2(mat1, mat1), matrix(lgl(), nrow = 0, ncol = 1))
   expect_equal(vec_ptype2(mat1, mat2), matrix(lgl(), nrow = 0, ncol = 2))
-  expect_error(vec_ptype2(mat2, mat3), "Incompatible")
+  expect_error(vec_ptype2(mat2, mat3), class = "vctrs_error_incompatible_type")
 })
 
-test_that("shape_match()", {
+test_that("vec_shaped_ptype()", {
   int <- function(...) array(NA_integer_, c(...))
 
-  expect_identical(shape_match(integer(), int(5), int(10)), new_shape(integer()))
-  expect_identical(shape_match(integer(), int(5, 1), int(10, 1)), new_shape(integer(), 1))
-  expect_identical(shape_match(integer(), int(5, 1, 2), int(10, 1, 2)), new_shape(integer(), 1:2))
+  expect_identical(vec_shaped_ptype(integer(), int(5), int(10)), new_shape(integer()))
+  expect_identical(vec_shaped_ptype(integer(), int(5, 1), int(10, 1)), new_shape(integer(), 1))
+  expect_identical(vec_shaped_ptype(integer(), int(5, 1, 2), int(10, 1, 2)), new_shape(integer(), 1:2))
 })
 
 # vec_cast() --------------------------------------------------------------
@@ -60,10 +63,12 @@ test_that("safe casts work as expected", {
   expect_equal(vec_cast(lgl(TRUE, FALSE), logical()), exp)
   expect_equal(vec_cast(int(1L, 0L), logical()), exp)
   expect_equal(vec_cast(dbl(1, 0), logical()), exp)
-  expect_equal(vec_cast(chr("T", "F"), logical()), exp)
-  expect_equal(vec_cast(chr("TRUE", "FALSE"), logical()), exp)
-  expect_equal(vec_cast(chr("true", "false"), logical()), exp)
-  expect_equal(vec_cast(list(1, 0), logical()), exp)
+
+  # These used to be allowed
+  expect_error(vec_cast(chr("T", "F"), logical()), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(chr("TRUE", "FALSE"), logical()), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(chr("true", "false"), logical()), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(list(1, 0), logical()), class = "vctrs_error_incompatible_type")
 })
 
 test_that("NA casts work as expected", {
@@ -73,8 +78,10 @@ test_that("NA casts work as expected", {
   expect_equal(vec_cast(lgl(NA), to), exp)
   expect_equal(vec_cast(int(NA), to), exp)
   expect_equal(vec_cast(dbl(NA), to), exp)
-  expect_equal(vec_cast(chr(NA), to), exp)
-  expect_equal(vec_cast(list(NA), to), exp)
+
+  # These used to be allowed
+  expect_error(vec_cast(chr(NA), to), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(list(NA), to), class = "vctrs_error_incompatible_type")
 })
 
 test_that("Shaped NA casts work as expected", {
@@ -85,21 +92,25 @@ test_that("Shaped NA casts work as expected", {
   expect_equal(vec_cast(mat(lgl(NA)), to_mat), exp_mat)
   expect_equal(vec_cast(mat(int(NA)), to_mat), exp_mat)
   expect_equal(vec_cast(mat(dbl(NA)), to_mat), exp_mat)
-  expect_equal(vec_cast(mat(chr(NA)), to_mat), exp_mat)
-  expect_equal(vec_cast(mat(list(NA)), to_mat), exp_mat)
+
+  # These used to be allowed
+  expect_error(vec_cast(mat(chr(NA)), to_mat), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(mat(list(NA)), to_mat), class = "vctrs_error_incompatible_type")
 })
 
 test_that("lossy casts generate warning", {
-  expect_lossy(vec_cast(int(2L, 1L), lgl()),                lgl(TRUE, TRUE), x = int(),  to = lgl())
-  expect_lossy(vec_cast(dbl(2, 1), lgl()),                  lgl(TRUE, TRUE), x = dbl(),  to = lgl())
-  expect_lossy(vec_cast(chr("x", "TRUE"), lgl()),           lgl(NA, TRUE),   x = chr(),  to = lgl())
-  expect_lossy(vec_cast(chr("t", "T"), lgl()),              lgl(NA, TRUE),   x = chr(),  to = lgl())
-  expect_lossy(vec_cast(chr("f", "F"), lgl()),              lgl(NA, FALSE),  x = chr(),  to = lgl())
-  expect_lossy(vec_cast(list(c(TRUE, FALSE), TRUE), lgl()), lgl(TRUE, TRUE), x = list(), to = lgl())
+  expect_lossy(vec_cast(int(2L, 1L), lgl()), lgl(TRUE, TRUE), x = int(),  to = lgl())
+  expect_lossy(vec_cast(dbl(2, 1), lgl()), lgl(TRUE, TRUE), x = dbl(),  to = lgl())
+
+  # These used to be allowed
+  expect_error(vec_cast(chr("x", "TRUE"), lgl()), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(chr("t", "T"), lgl()), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(chr("f", "F"), lgl()), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(list(c(TRUE, FALSE), TRUE), lgl()), class = "vctrs_error_incompatible_type")
 })
 
 test_that("invalid casts generate error", {
-  expect_error(vec_cast(factor("a"), logical()), class = "vctrs_error_incompatible_cast")
+  expect_error(vec_cast(factor("a"), logical()), class = "vctrs_error_incompatible_type")
 })
 
 test_that("dimensionality matches output" ,{
@@ -109,16 +120,14 @@ test_that("dimensionality matches output" ,{
   expect_dim(vec_cast(TRUE, x2), c(1, 2))
 
   x <- matrix(1, nrow = 2, ncol = 2)
-  expect_error(vec_cast(x, logical()), class = "vctrs_error_incompatible_cast")
+  expect_error(vec_cast(x, logical()), class = "vctrs_error_incompatible_type")
 })
 
 test_that("the common type of two `NA` vectors is unspecified", {
   expect_equal(vec_ptype2(NA, NA), unspecified())
-
-  # Ensure the R level dispatch is consistent
-  expect_equal(vec_default_ptype2(NA, NA), unspecified())
 })
 
+
 # Integer
 
 test_that("safe casts work as expected", {
@@ -126,8 +135,10 @@ test_that("safe casts work as expected", {
   expect_equal(vec_cast(lgl(TRUE, FALSE), integer()), int(1L, 0L))
   expect_equal(vec_cast(int(1L, 2L), integer()), int(1L, 2L))
   expect_equal(vec_cast(dbl(1, 2), integer()), int(1L, 2L))
-  expect_equal(vec_cast(chr("1", "2"), integer()), int(1L, 2L))
-  expect_equal(vec_cast(list(1L, 2L), integer()), int(1L, 2L))
+
+  # These used to be allowed
+  expect_error(vec_cast(chr("1", "2"), integer()), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(list(1L, 2L), integer()), class = "vctrs_error_incompatible_type")
 })
 
 test_that("NA casts work as expected", {
@@ -137,8 +148,10 @@ test_that("NA casts work as expected", {
   expect_equal(vec_cast(lgl(NA), to), exp)
   expect_equal(vec_cast(int(NA), to), exp)
   expect_equal(vec_cast(dbl(NA), to), exp)
-  expect_equal(vec_cast(chr(NA), to), exp)
-  expect_equal(vec_cast(list(NA), to), exp)
+
+  # These used to be allowed
+  expect_error(vec_cast(chr(NA), to), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(list(NA), to), class = "vctrs_error_incompatible_type")
 })
 
 test_that("Shaped NA casts work as expected", {
@@ -149,20 +162,23 @@ test_that("Shaped NA casts work as expected", {
   expect_equal(vec_cast(mat(lgl(NA)), to_mat), exp_mat)
   expect_equal(vec_cast(mat(int(NA)), to_mat), exp_mat)
   expect_equal(vec_cast(mat(dbl(NA)), to_mat), exp_mat)
-  expect_equal(vec_cast(mat(chr(NA)), to_mat), exp_mat)
-  expect_equal(vec_cast(mat(list(NA)), to_mat), exp_mat)
+
+  # These used to be allowed
+  expect_error(vec_cast(mat(chr(NA)), to_mat), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(mat(list(NA)), to_mat), class = "vctrs_error_incompatible_type")
 })
 
 test_that("lossy casts generate error", {
   expect_lossy(vec_cast(c(2.5, 2), int()),     int(2, 2), x = dbl(), to = int())
-  expect_lossy(vec_cast(c("2.5", "2"), int()), int(2, 2), x = chr(), to = int())
-
   expect_lossy(vec_cast(c(.Machine$integer.max + 1, 1), int()),  int(NA, 1L), x = dbl(), to = int())
   expect_lossy(vec_cast(c(-.Machine$integer.max - 1, 1), int()), int(NA, 1L), x = dbl(), to = int())
+
+  # These used to be allowed
+  expect_error(vec_cast(c("2.5", "2"), int()), class = "vctrs_error_incompatible_type")
 })
 
 test_that("invalid casts generate error", {
-  expect_error(vec_cast(factor("a"), integer()), class = "vctrs_error_incompatible_cast")
+  expect_error(vec_cast(factor("a"), integer()), class = "vctrs_error_incompatible_type")
 })
 
 
@@ -173,8 +189,10 @@ test_that("safe casts work as expected", {
   expect_equal(vec_cast(lgl(TRUE, FALSE), double()), dbl(1, 0))
   expect_equal(vec_cast(int(1, 0), double()), dbl(1, 0))
   expect_equal(vec_cast(dbl(1, 1.5), double()), dbl(1, 1.5))
-  expect_equal(vec_cast(chr("1", "1.5"), double()), dbl(1, 1.5))
-  expect_equal(vec_cast(list(1, 1.5), double()), dbl(1, 1.5))
+
+  # These used to be allowed
+  expect_error(vec_cast(chr("1", "1.5"), double()), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(list(1, 1.5), double()), class = "vctrs_error_incompatible_type")
 })
 
 test_that("NA casts work as expected", {
@@ -184,8 +202,10 @@ test_that("NA casts work as expected", {
   expect_equal(vec_cast(lgl(NA), to), exp)
   expect_equal(vec_cast(int(NA), to), exp)
   expect_equal(vec_cast(dbl(NA), to), exp)
-  expect_equal(vec_cast(chr(NA), to), exp)
-  expect_equal(vec_cast(list(NA), to), exp)
+
+  # These used to be allowed
+  expect_error(vec_cast(chr(NA), to), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(list(NA), to), class = "vctrs_error_incompatible_type")
 })
 
 test_that("Shaped NA casts work as expected", {
@@ -196,16 +216,14 @@ test_that("Shaped NA casts work as expected", {
   expect_equal(vec_cast(mat(lgl(NA)), to_mat), exp_mat)
   expect_equal(vec_cast(mat(int(NA)), to_mat), exp_mat)
   expect_equal(vec_cast(mat(dbl(NA)), to_mat), exp_mat)
-  expect_equal(vec_cast(mat(chr(NA)), to_mat), exp_mat)
-  expect_equal(vec_cast(mat(list(NA)), to_mat), exp_mat)
-})
 
-test_that("lossy casts generate warning", {
-  expect_lossy(vec_cast(c("2.5", "x"), dbl()), dbl(2.5, NA), x = chr(), to = dbl())
+  # These used to be allowed
+  expect_error(vec_cast(mat(chr(NA)), to_mat), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(mat(list(NA)), to_mat), class = "vctrs_error_incompatible_type")
 })
 
 test_that("invalid casts generate error", {
-  expect_error(vec_cast(factor("a"), double()), class = "vctrs_error_incompatible_cast")
+  expect_error(vec_cast(factor("a"), double()), class = "vctrs_error_incompatible_type")
 })
 
 
@@ -216,7 +234,9 @@ test_that("safe casts to complex works", {
   expect_identical(vec_cast(lgl(TRUE, FALSE), cpl()), cpl(1, 0))
   expect_identical(vec_cast(int(1, 0), cpl()), cpl(1, 0))
   expect_identical(vec_cast(dbl(1, 1.5), cpl()), cpl(1, 1.5))
-  expect_identical(vec_cast(list(1, 1.5), cpl()), cpl(1, 1.5))
+
+  # This used to be allowed
+  expect_error(vec_cast(list(1, 1.5), cpl()), class = "vctrs_error_incompatible_type")
 })
 
 test_that("NA casts work as expected", {
@@ -226,7 +246,9 @@ test_that("NA casts work as expected", {
   expect_equal(vec_cast(lgl(NA), to), exp)
   expect_equal(vec_cast(int(NA), to), exp)
   expect_equal(vec_cast(dbl(NA), to), exp)
-  expect_equal(vec_cast(list(NA), to), exp)
+
+  # This used to be allowed
+  expect_error(vec_cast(list(NA), to), class = "vctrs_error_incompatible_type")
 })
 
 test_that("Shaped NA casts work as expected", {
@@ -237,7 +259,9 @@ test_that("Shaped NA casts work as expected", {
   expect_equal(vec_cast(mat(lgl(NA)), to_mat), exp_mat)
   expect_equal(vec_cast(mat(int(NA)), to_mat), exp_mat)
   expect_equal(vec_cast(mat(dbl(NA)), to_mat), exp_mat)
-  expect_equal(vec_cast(mat(list(NA)), to_mat), exp_mat)
+
+  # This used to be allowed
+  expect_error(vec_cast(mat(list(NA)), to_mat), class = "vctrs_error_incompatible_type")
 })
 
 test_that("complex is coercible to numeric types", {
@@ -266,8 +290,10 @@ test_that("complex is not coercible to logical", {
 test_that("safe casts work as expected", {
   expect_equal(vec_cast(NULL, character()), NULL)
   expect_equal(vec_cast(NA, character()), NA_character_)
-  expect_equal(vec_cast(lgl(TRUE, FALSE), character()), chr("TRUE", "FALSE"))
-  expect_equal(vec_cast(list("x", "y"), character()), chr("x", "y"))
+
+  # These used to be allowed
+  expect_error(vec_cast(lgl(TRUE, FALSE), character()), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(list("x", "y"), character()), class = "vctrs_error_incompatible_type")
 })
 
 test_that("NA casts work as expected", {
@@ -275,10 +301,12 @@ test_that("NA casts work as expected", {
   to <- chr()
 
   expect_equal(vec_cast(lgl(NA), to), exp)
-  expect_equal(vec_cast(int(NA), to), exp)
-  expect_equal(vec_cast(dbl(NA), to), exp)
   expect_equal(vec_cast(chr(NA), to), exp)
-  expect_equal(vec_cast(list(NA), to), exp)
+
+  # These used to be allowed
+  expect_error(vec_cast(int(NA), to), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(dbl(NA), to), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(list(NA), to), class = "vctrs_error_incompatible_type")
 })
 
 test_that("Shaped NA casts work as expected", {
@@ -286,16 +314,19 @@ test_that("Shaped NA casts work as expected", {
   exp_mat <- mat(chr(NA))
   to_mat <- matrix(chr())
 
-  expect_equal(vec_cast(mat(lgl(NA)), to_mat), exp_mat)
-  expect_equal(vec_cast(mat(int(NA)), to_mat), exp_mat)
-  expect_equal(vec_cast(mat(dbl(NA)), to_mat), exp_mat)
   expect_equal(vec_cast(mat(chr(NA)), to_mat), exp_mat)
-  expect_equal(vec_cast(mat(list(NA)), to_mat), exp_mat)
+
+  # These used to be allowed
+  expect_error(vec_cast(mat(lgl(NA)), to_mat), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(mat(int(NA)), to_mat), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(mat(dbl(NA)), to_mat), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(mat(list(NA)), to_mat), class = "vctrs_error_incompatible_type")
 })
 
-test_that("difftime gets special treatment", {
+test_that("difftime does not get special treatment", {
   dt1 <- as.difftime(600, units = "secs")
-  expect_equal(vec_cast(dt1, character()), "600 secs")
+  # This used to be allowed
+  expect_error(vec_cast(dt1, character()), class = "vctrs_error_incompatible_type")
 })
 
 
@@ -303,12 +334,14 @@ test_that("difftime gets special treatment", {
 
 test_that("safe casts work as expected", {
   expect_equal(vec_cast(NULL, raw()), NULL)
-  expect_equal(vec_cast(list(raw(1)), raw()), raw(1))
+
+  # This used to be allowed
+  expect_error(vec_cast(list(raw(1)), raw()), class = "vctrs_error_incompatible_type")
 })
 
 test_that("invalid casts generate error", {
-  expect_error(vec_cast(raw(1), double()), class = "vctrs_error_incompatible_cast")
-  expect_error(vec_cast(double(1), raw()), class = "vctrs_error_incompatible_cast")
+  expect_error(vec_cast(raw(1), double()), class = "vctrs_error_incompatible_type")
+  expect_error(vec_cast(double(1), raw()), class = "vctrs_error_incompatible_type")
 })
 
 test_that("can sort raw", {
@@ -322,19 +355,31 @@ test_that("raw has informative type summaries", {
   expect_equal(vec_ptype_full(raw()), "raw")
 })
 
+test_that("can provide common type with raw", {
+  local_methods(
+    vec_ptype2.raw.vctrs_foobar = function(...) "dispatched-left",
+    vec_ptype2.vctrs_foobar = function(...) NULL,
+    vec_ptype2.vctrs_foobar.raw = function(...) "dispatched-right"
+  )
+  expect_identical(vec_ptype2(raw(), foobar("")), "dispatched-left")
+  expect_identical(vec_ptype2(foobar(""), raw()), "dispatched-right")
+})
+
 
 # Lists
 
 test_that("safe casts work as expected", {
   expect_equal(vec_cast(NULL, list()), NULL)
   expect_equal(vec_cast(NA, list()), list(NULL))
-  expect_equal(vec_cast(1:2, list()), list(1L, 2L))
   expect_equal(vec_cast(list(1L, 2L), list()), list(1L, 2L))
+
+  # This used to be allowed
+  expect_error(vec_cast(1:2, list()), class = "vctrs_error_incompatible_type")
 })
 
 test_that("dimensionality matches to" ,{
   x1 <- matrix(TRUE, nrow = 1, ncol = 1)
-  x2 <- matrix(list(), nrow = 0, ncol = 2)
+  x2 <- matrix(1L, nrow = 0, ncol = 2)
   expect_dim(vec_cast(x1, x2), c(1, 2))
   expect_dim(vec_cast(TRUE, x2), c(1, 2))
 })
@@ -342,7 +387,9 @@ test_that("dimensionality matches to" ,{
 test_that("data frames are cast to list row wise (#639)", {
   x <- data.frame(x = 1:2, row.names = c("a", "b"))
   expect <- list(data.frame(x = 1L), data.frame(x = 2L))
-  expect_equal(vec_cast(x, list()), expect)
+
+  # This used to be allowed
+  expect_error(vec_cast(x, list()), class = "vctrs_error_incompatible_type")
 })
 
 test_that("data frames can be cast to shaped lists", {
@@ -352,19 +399,24 @@ test_that("data frames can be cast to shaped lists", {
   expect <- list(vec_slice(x, 1), vec_slice(x, 2))
   expect <- array(expect, dim = c(2, 2, 1))
 
-  expect_equal(vec_cast(x, to), expect)
+  # This used to be allowed
+  expect_error(vec_cast(x, to), class = "vctrs_error_incompatible_type")
 })
 
 test_that("Casting atomic `NA` values to list results in a `NULL`", {
   x <- c(NA, 1)
   expect <- list(NULL, 1)
-  expect_equal(vec_cast(x, list()), expect)
+
+  # This used to be allowed
+  expect_error(vec_cast(x, list()), class = "vctrs_error_incompatible_type")
 })
 
 test_that("Casting data frame `NA` rows to list results in a `NULL`", {
   x <- data.frame(x = c(NA, NA, 1), y = c(NA, 1, 2))
   expect <- list(NULL, vec_slice(x, 2), vec_slice(x, 3))
-  expect_equal(vec_cast(x, list()), expect)
+
+  # This used to be allowed
+  expect_error(vec_cast(x, list()), class = "vctrs_error_incompatible_type")
 })
 
 
